@@ -352,8 +352,11 @@ class AbstractOverriddenElementModification(MutationOperator):
                 raise MutationResign()
             parent = parent.parent
         getattr_rec = lambda obj, attr: functools.reduce(getattr, attr, obj)
-        klass = getattr_rec(self.module, reversed(parent_names))
-        for base_klass in klass.mro()[1:-1]:
+        try:
+            klass = getattr_rec(self.module, reversed(parent_names))
+        except AttributeError:
+            raise MutationResign()
+        for base_klass in type.mro(klass)[1:-1]:
             if hasattr(base_klass, name):
                 return True
         return False
@@ -377,7 +380,7 @@ class HidingVariableDeletion(AbstractOverriddenElementModification):
         new_targets = []
         new_values = []
         for target_element, value_element in zip(target.elts, value.elts):
-            if not self.is_overridden(node, target_element.id):
+            if not self.is_overridden(node, getattr(target_element, 'id', None)):
                 new_targets.append(target_element)
                 new_values.append(value_element)
         if len(new_targets) == len(target.elts):
